<?php

namespace ipl\Orm;

/**
 * Interface Behavior
 *
 * @internal Used for type hinting only. Concrete behaviors are supposed to implement contracts from ipl\Orm\Contract
 */
interface Behavior
{
}
