.empty-state {
  color: var(--empty-state-color, @empty-state-color);
}

.empty-state-bar {
  padding: 1em;
  text-align: center;

  .rounded-corners();
  background-color: var(--empty-state-bar-bg, @empty-state-bar-bg);
  color: var(--default-text-color, @default-text-color);
}
