.action-link {
  color: var(--control-color, @control-color);
}

.button-link {
  .action-link();
  .rounded-corners(3px);

  background: var(--default-input-bg, @default-input-bg);
  display: inline-block;
  padding: 0.25em 0.5em;

  &:hover {
    background: var(--default-input-hover-bg, @default-input-hover-bg);
    text-decoration: none;
  }

  &[aria-disabled="true"] {
    background: none;
    border: 1px solid var(--control-disabled-color, @control-disabled-color);
    color: var(--control-disabled-color, @control-disabled-color);
    cursor: not-allowed;

    .user-select(none);
  }
}
